@@ -1,5 +1,6 @@
 .onAttach <- function(libname, pkgname) {
-  packageStartupMessage("
+  version <- utils::packageVersion("MOSS")
+  pkg_message <- paste0("
  _____________________________________________________________________
 |MOSS: Multi-Omic integration via Sparse Singular value decomposition.|
  _____________________________________________________________________
@@ -9,8 +10,11 @@ Agustin Gonzalez-Reymundez, Alexander Grueneberg, and Ana I. Vazquez.
 Maintainer: Agustin Gonzalez-Reymundez <agugonrey@gmail.com>
 URL: https://github.com/agugonrey/MOSS
 BugReports: https://github.com/agugonrey/MOSS/issues
+  
+Version ",version,
 
-Type 'help(moss)' for package overview and examples.
+".\n\nType 'help(moss)' for package overview and examples.
 Type 'sim.data <- simulate_data()' to generate a small simulated data set.
 ")
+  packageStartupMessage(pkg_message)
 }
